#ifndef _FIRECAM_
#define _FIRECAM_


// Video formats
#define FORMAT_VGA	0
#define	FORMAT_SVGA1	1
#define FORMAT_SVGA2	2

// Video modes
#define MODE_160_120_YUV444	0
#define MODE_320_240_YUV422	1
#define MODE_640_480_YUV411	2
#define MODE_640_480_YUV422	3
#define MODE_640_480_RGB	4
#define MODE_640_480_MONO8	5
#define MODE_640_480_MONO16	6

// Video framerates
#define RATE_7_5	2
#define RATE_15		3
#define RATE_30		4

// Feature elements
#define ELEM_BRIGHTNESS		0
#define ELEM_AUTO_EXPOSURE	1
#define ELEM_SHARPNESS		2
#define ELEM_WHITE_BALANCE	3
#define ELEM_HUE		4
#define ELEM_SATURATION		5
#define ELEM_GAMMA		6
#define ELEM_SHUTTER		7
#define ELEM_GAIN		8
#define ELEM_IRIS		9
#define ELEM_FOCUS		10
#define ELEM_TEMPERATURE	11
#define ELEM_TRIGGER		12
#define ELEM_ZOOM		32
#define ELEM_PAN		33
#define ELEM_TILT		34
#define ELEM_FILTER		35

// Trigger modes
#define TRIGGER_MODE_1 0x000
#define TRIGGER_MODE_2 0x010
#define TRIGGER_MODE_3 0x020
#define TRIGGER_MODE_4 0x030


typedef struct
{
	int	available;
	int	auto_control;
	int	man_control;
	int	on_off;
	int	min_value;
	int	max_value;
} elem_query_t;

typedef struct
{
	int	on_off;
	int	auto_control;
	int	value;
	int	value2;
} elem_control_t;

// Call before using any of the other functions
int	firecam_open();

// Call only once after TM4 has been programmed
int	firecam_init();

// This function is not really needed.
// It determines how many cameras are actually connected.
int	firecam_enumerate();

// Resets a camera's settings to default
int	firecam_reset(int cam_id);

// Check for presence of format/mode/framerate
int	firecam_format_available(int cam_id, int format);
int	firecam_mode_available(int cam_id, int format, int mode);
int	firecam_rate_available(int cam_id, int format, int mode, int rate);

// Set format/mode/frameraet
int	firecam_set_format(int cam_id, int format);
int	firecam_set_mode(int cam_id, int mode);
int	firecam_set_rate(int cam_id, int rate);

// Control frame transmission
int	firecam_start_streaming(int cam_id, int speed);
int	firecam_stop_streaming(int cam_id);
int	firecam_is_streaming(int cam_id);

// Query/set camera picture elements
int	firecam_query_element(int cam_id, int element_id, elem_query_t* out);
int	firecam_get_elem_status(int cam_id, int element_id, elem_control_t* out);
int	firecam_set_elem_status(int cam_id, int element_id, elem_control_t* in);

// Set by firecam_enumerate()
extern int firecam_num_cameras;


#endif
