library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- Has 4 cycles of latency

entity yuv_to_rgb is
port
(
	clk	: in	std_logic;

	y_in	: in	std_logic_vector(7 downto 0);
	u_in	: in	std_logic_vector(7 downto 0);
	v_in	: in	std_logic_vector(7 downto 0);

	r_out	: out	std_logic_vector(7 downto 0);
	g_out	: out	std_logic_vector(7 downto 0);
	b_out	: out	std_logic_vector(7 downto 0)
);
end yuv_to_rgb;


architecture comb of yuv_to_rgb is
	signal y_tmp, u_tmp, v_tmp : integer;
	signal r_tmp, g_tmp, b_tmp : integer;
	signal v1, u1, v2, u2 : integer;
begin
	process (clk)
	begin
		if rising_edge(clk) then
			y_tmp <= conv_integer(y_in);
			u_tmp <= conv_integer(u_in) - 128;
			v_tmp <= conv_integer(v_in) - 128;

			v1 <= 359*v_tmp;
			u1 <= 88*u_tmp;
			v2 <= 183*v_tmp;
			u2 <= 454*u_tmp;

			r_tmp <= y_tmp + v1/256;
			g_tmp <= y_tmp - (u1 + v2)/256;
			b_tmp <= y_tmp + u2/256;

			if r_tmp > 255 then
				r_out <= "11111111";
			elsif r_tmp < 0 then
				r_out <= "00000000";
			else
				r_out <= conv_std_logic_vector(r_tmp, 8);
			end if;

			if g_tmp > 255 then
				g_out <= "11111111";
			elsif g_tmp < 0 then
				g_out <= "00000000";
			else
				g_out <= conv_std_logic_vector(g_tmp, 8);
			end if;

			if b_tmp > 255 then
				b_out <= "11111111";
			elsif b_tmp < 0 then
				b_out <= "00000000";
			else
				b_out <= conv_std_logic_vector(b_tmp, 8);
			end if;
		end if;
	end process;
end comb;
